DIR_UNKNOWN = 1
DIR_APPROACHING = 2
DIR_RECEDING = 3


class Fact:
    def __init__(self, args_jnode, args_store, arch_files, feed_files):
        """
        :type args_jnode: dict
        :type args_store: str
        :type arch_files: list[dict]
        :type feed_files: dict
        """
        self.jnode = args_jnode
        self.args_store = args_store
        self.arch_files = arch_files
        self.feed_files = feed_files

    @staticmethod
    def unpack(args_name, args_jnode):
        """
        :type args_jnode: dict
        :type args_name: str

        :rtype: Fact
        :raise ValueError:
        """
        feed_files = {}
        arch_files = []

        def add_arch_file(name, altref=None):
            if altref:
                arch_files.append({
                    'ref': name,
                    'name': name,
                    'altref': altref
                })
            else:
                arch_files.append({
                    'ref': name,
                    'name': name
                })

        # args.json
        args_blob_ref = args_name.split('-')[-1]
        feed_files['args'] = args_blob_ref
        add_arch_file(args_blob_ref, 'args.json')

        def analyze_measurements():
            measurements = args_jnode['measurements']

            for measurement in measurements:
                waypoints = measurement['waypoints']
                best_images = waypoints['best'].get('images', {})
                plate_images = best_images.get('license-plates', [])

                # thumb.jpeg
                thumb_im_name = best_images['thumb']['name']
                feed_files['thumb'] = thumb_im_name
                add_arch_file(thumb_im_name, 'thumb.jpeg')

                # scene.jpeg
                scene_im_name = best_images['scene']['name']
                add_arch_file(scene_im_name, 'scene.jpeg')
                feed_files['scene'] = scene_im_name

                # vehicle.jpeg
                vehicle_im_name = best_images['vehicle']['name']
                add_arch_file(vehicle_im_name, 'vehicle.jpeg')
                feed_files['vehicle'] = vehicle_im_name

                # plate.jpeg
                map(lambda plate_im: add_arch_file(plate_im['name'], 'plate.jpeg'), plate_images)

            plate_images = measurements[0]['waypoints']['best'].get('images', {}).get('license-plates', [])
            if len(plate_images):
                feed_files['plate'] = plate_images[0]['name']

        def analyze_evidence():
            e_jnode = args_jnode.get('evidence', {})
            em_jnode = e_jnode.get('measurements')
            if not em_jnode:
                return

            emw_jnodes = em_jnode.get('waypoints', {})
            for waypoint in emw_jnodes.itervalues():
                if type(waypoint) != list:
                    waypoint = [waypoint]

                for w_jnode in waypoint:
                    wi_jnodes = w_jnode.get('images', {})
                    for wi_jnode in wi_jnodes.itervalues():
                        add_arch_file(wi_jnode['name'])

            emv_jnodes = em_jnode.get('videos', [])
            for emv_jnode in emv_jnodes:
                add_arch_file(emv_jnode['name'])

        analyze_measurements()
        analyze_evidence()

        bs_chan = args_blob_ref.split('_')[0]
        return Fact(args_jnode, bs_chan, arch_files, feed_files)
